@@ -4,30 +4,33 @@ from .timings import CancleSubscription, ThreadJob
 from .helpers import *
 import socketio
 from inspect import signature
-from typing import overload, cast, Any, Union, Literal, Callable, List, Optional, Any, Final
+from typing import overload, cast, Any, Union, Literal, Callable, List, Optional, Any
 from copy import deepcopy
 from itertools import repeat
 from .types import *
 from .colors import Colors
 from random import randint
-
-
-class INPUT_TYPE:
-    TEXT: Final[str] = 'text'
-    NUMBER: Final[str] = 'number'
-    DATETIME: Final[str] = 'datetime'
-    DATE: Final[str] = 'date'
-    TIME: Final[str] = 'time'
-    SELECT: Final[str] = 'select'
+from contextlib import contextmanager
 
 
 def noop(x):
     pass
 
 
+DATA_MSG_THRESHOLD = 5
+CHARTEABLE_DATA_MSG_THRESHOLD = 120  # around 2 seconds @ 16ms
+
+
+def data_threshold(data_type: str) -> int:
+    if data_type == DataType.ACCELERATION or data_type == DataType.GYRO:
+        return CHARTEABLE_DATA_MSG_THRESHOLD
+    return DATA_MSG_THRESHOLD
+
+
 class Connector:
     __last_time_stamp: float = -1
     __last_sub_time: float = 0
+    __record_data__: bool = False
     data: dict[str, dict[str, list[ClientMsg]]] = DictX({})
     __devices = {'time_stamp': time_s(), 'devices': []}
     device: Optional[Device] = None
@@ -441,7 +444,7 @@ class Connector:
         ...
 
     @overload
-    def all_data(self, data_type: DataType, device_id: str) -> List[ClientMsg]:
+    def all_data(self, data_type: DataType = None, device_id: str = None) -> List[ClientMsg]:
         ...
 
     def all_data(self, data_type: str = None, device_id: str = None) -> List[ClientMsg]:
@@ -564,28 +567,228 @@ class Connector:
     def latest_key(self, device_id: str = '__ALL_DEVICES__') -> Union[None, KeyMsg]:
         return self.latest_data('key', device_id=device_id)
 
-    def configure_playground(self, config: Union[dict, PlaygroundConfig], **delivery_opts):
-        self.emit(SocketEvents.NEW_DATA, {
-            'type': 'playground_config',
-            'config': config
-        }, **delivery_opts)
+    def configure_playground(self,
+                             width: Optional[Number] = None,
+                             height: Optional[Number] = None,
+                             shift_x: Optional[Number] = None,
+                             shift_y: Optional[Number] = None,
+                             color: Optional[Union[Colors, str]] = None,
+                             **delivery_opts):
+        config = {
+            'type': DataType.PLAYGROUND_CONFIG,
+            'config': without_none({
+                'width': width,
+                'height': height,
+                'shift_x': shift_x,
+                'shift_y': shift_y,
+                'color': color
+            })
+        }
+        self.emit(SocketEvents.NEW_DATA, config, **delivery_opts)
 
-    def add_sprites(self, sprites: List[Union[dict, dict]], **delivery_opts):
-        self.__sprites.extend(map(lambda sprite: DictX(sprite), sprites))
-        self.emit(
-            SocketEvents.NEW_DATA,
-            {
-                'type': DataType.SPRITES,
-                'sprites': sprites
-            },
-            **delivery_opts
-        )
+    @contextmanager
+    def add_sprites(self, **delivery_opts):
+        sprites = []
 
-    def add_sprite(self, sprite: dict, **delivery_opts):
-        self.__sprites.append(DictX(sprite))
-        if 'id' not in sprite:
-            sprite['id'] = f'sprite{randint(10000, 99999)}'
+        def sprite(
+                id: Optional[str],
+                clickable: Optional[bool] = None,
+                collision_detection: Optional[bool] = None,
+                color: Optional[Union[Colors, str]] = None,
+                direction: Optional[List[Number]] = None,
+                distance: Optional[Number] = None,
+                form: Optional[Union[SpriteForm, Literal['round', 'rectangle']]] = None,
+                height: Optional[Number] = None,
+                pos_x: Optional[Number] = None,
+                pos_y: Optional[Number] = None,
+                reset_time: Optional[Number] = None,
+                speed: Optional[Number] = None,
+                text: Optional[str] = None,
+                time_span: Optional[Number] = None,
+                width: Optional[Number] = None,
+                **delivery_opts):
+            '''
+            Optional
+            --------
+            id : str
+                unique id. When already present, this sprite will be updated.
 
+            form : 'round' | 'rectangle'
+                the form of the sprite
+
+            width : Number
+                the width of the sprite in playground units
+
+            height : Number
+                the height of the sprite in playground units
+
+            pos_x : Number
+                the x position in playground units
+
+            pos_y : Number
+                the y position in playground units
+
+            color : str
+                the color of the sprite
+
+            clickable : bool
+                wheter an event is delivered when the sprite is clicked
+
+            collision_detection : bool
+                wheter to report collisions with other sprites or not. Collision-Detection can be costly,
+                so use it with care.
+
+            direction : [x: Number, y: Number]
+                the direction of the sprite. In combination with a speed value, the sprite will be advanced
+                for each display refresh in this direction.
+
+            distance : Number
+                the sprite will disappear after the given distance (in auto-movement mode) is reached.
+
+            reset_time : bool
+                wheter to reset the start time on an update. Resets the time_span or distance
+                for auto-movement sprites.
+
+            speed : Number
+                makes a sprite auto-movable.
+
+            text : str
+                the text that is displayed on the sprite.
+
+            time_span : Number
+                the time a sprite lives
+            '''
+            s = {
+                'id': id if id is not None else f'sprite{randint(10000, 99999)}',
+                'clickable': clickable,
+                'collision_detection': collision_detection,
+                'color': color,
+                'direction': direction,
+                'distance': distance,
+                'form': form,
+                'height': height,
+                'pos_x': pos_x,
+                'pos_y': pos_y,
+                'reset_time': reset_time,
+                'speed': speed,
+                'text': text,
+                'time_span': time_span,
+                'width': width
+            }
+            sprites.append(without_none(s))
+        try:
+            yield sprite
+        except:
+            sprites = []
+            raise
+        else:
+            for s in sprites:
+                to_update = first(lambda s: s['id'] == s['id'], self.__sprites)
+                if to_update is not None:
+                    to_update.update(s)
+                else:
+                    self.__sprites.append(DictX(s))
+            self.emit(
+                SocketEvents.NEW_DATA,
+                {
+                    'type': DataType.SPRITES,
+                    'sprites': sprites
+                },
+                **delivery_opts
+            )
+
+    def add_sprite(
+            self,
+            id: Optional[str],
+            clickable: Optional[bool] = None,
+            collision_detection: Optional[bool] = None,
+            color: Optional[Union[Colors, str]] = None,
+            direction: Optional[List[Number]] = None,
+            distance: Optional[Number] = None,
+            form: Optional[Union[SpriteForm, Literal['round', 'rectangle']]] = None,
+            height: Optional[Number] = None,
+            pos_x: Optional[Number] = None,
+            pos_y: Optional[Number] = None,
+            reset_time: Optional[Number] = None,
+            speed: Optional[Number] = None,
+            text: Optional[str] = None,
+            time_span: Optional[Number] = None,
+            width: Optional[Number] = None,
+            **delivery_opts):
+        '''
+        Optional
+        --------
+        id : str
+            unique id. When already present, this sprite will be updated.
+
+        form : 'round' | 'rectangle'
+            the form of the sprite
+
+        width : Number
+            the width of the sprite in playground units
+
+        height : Number
+            the height of the sprite in playground units
+
+        pos_x : Number
+            the x position in playground units
+
+        pos_y : Number
+            the y position in playground units
+
+        color : str
+            the color of the sprite
+
+        clickable : bool
+            wheter an event is delivered when the sprite is clicked
+
+        collision_detection : bool
+            wheter to report collisions with other sprites or not. Collision-Detection can be costly,
+            so use it with care.
+
+        direction : [x: Number, y: Number]
+            the direction of the sprite. In combination with a speed value, the sprite will be advanced
+            for each display refresh in this direction.
+
+        distance : Number
+            the sprite will disappear after the given distance (in auto-movement mode) is reached.
+
+        reset_time : bool
+            wheter to reset the start time on an update. Resets the time_span or distance
+            for auto-movement sprites.
+
+        speed : Number
+            makes a sprite auto-movable.
+
+        text : str
+            the text that is displayed on the sprite.
+
+        time_span : Number
+            the time a sprite lives
+        '''
+        sprite = {
+            'id': id if id is not None else f'sprite{randint(10000, 99999)}',
+            'clickable': clickable,
+            'collision_detection': collision_detection,
+            'color': color,
+            'direction': direction,
+            'distance': distance,
+            'form': form,
+            'height': height,
+            'pos_x': pos_x,
+            'pos_y': pos_y,
+            'reset_time': reset_time,
+            'speed': speed,
+            'text': text,
+            'time_span': time_span,
+            'width': width
+        }
+        sprite = without_none(sprite)
+        to_update = first(lambda s: s['id'] == sprite['id'], self.__sprites)
+        if to_update is not None:
+            to_update.update(sprite)
+        else:
+            self.__sprites.append(DictX(sprite))
         self.emit(
             SocketEvents.NEW_DATA,
             {
@@ -595,6 +798,8 @@ class Connector:
             **delivery_opts
         )
 
+    update_sprites = add_sprites
+
     def clear_playground(self, **delivery_opts):
         self.__sprites.clear()
         self.emit(
@@ -619,20 +824,7 @@ class Connector:
             **delivery_opts
         )
 
-    def update_sprite(self, id: str, update: Union[dict, UpdateSprite], **delivery_opts):
-        sprite = first(lambda s: s.id == id, self.__sprites)
-        if sprite:
-            sprite.update(update)
-        update.update({'id': id})
-
-        self.emit(
-            SocketEvents.NEW_DATA,
-            {
-                'type': 'sprite',
-                'sprite': update
-            },
-            **delivery_opts
-        )
+    update_sprite = add_sprite
 
     @property
     def get_grid(self) -> List[List[CssColorType]]:
@@ -963,6 +1155,17 @@ class Connector:
     def leave_room(self, device_id: str):
         self.emit(SocketEvents.LEAVE_ROOM, DictX({'room': device_id}))
 
+    def start_recording(self):
+        self.clean_data()
+        self.__record_data__ = True
+
+    def stop_recording(self):
+        self.__record_data__ = False
+
+    @property
+    def is_recording(self):
+        return self.__record_data__
+
     def __on_connect(self):
         logging.info('SocketIO connected')
 
@@ -998,7 +1201,12 @@ class Connector:
         if data['type'] not in self.data[data['device_id']]:
             self.data[data['device_id']][data['type']] = []
 
+        data_len = len(self.data[data['device_id']][data['type']])
+        if not self.__record_data__ and (data_len >= data_threshold(data['type'])):
+            self.data[data['device_id']][data['type']].pop(0)
+
         self.data[data['device_id']][data['type']].append(cast(ClientMsg, data))
+
         if self.__main_thread_blocked:
             self.__blocked_data_msgs.append(cast(DataMsg, data))
         else:
