@@ -1,9 +1,20 @@
 from __future__ import annotations
-from typing import Final, overload, Union, Literal, Optional, Tuple, List, TypedDict
+from typing import overload, Union, Literal, Optional, Tuple, List, TypedDict
 from dataclasses import dataclass
 from .dictx import DictX
 from enum import Enum
 
+Number = Union[float, int]
+
+
+class InputType(str, Enum):
+    TEXT = 'text'
+    NUMBER = 'number'
+    DATETIME = 'datetime'
+    DATE = 'date'
+    TIME = 'time'
+    SELECT = 'select'
+
 
 class SocketEvents(str, Enum):
     ALL_DATA = "all_data"
@@ -337,6 +348,7 @@ class PlaygroundConfig(DataMsg):
     height: Optional[int]
     shift_x: Optional[int]
     shift_y: Optional[int]
+    color: Optional[str]
 
 
 class PlaygroundConfigMsg(PlaygroundConfig):
